/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AskPassQt.h"
#include "AskPassQt.moc"

#include "AnimatedLineEditor.h"
#include "CompositeEngine.h"
#include "RoundedRegion.h"
#include "ShadowLabel.h"
#include "SvgEngine.h"
#include "WinUtil.h"

#include <QFile>
#include <QApplication>
#include <QDesktopWidget>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QPainter>

#include <cstdio>
#include <iostream>

//______________________________________________________
AskPassQt::AskPassQt( QWidget* parent ) :
    Transparency::TransparentWidget( parent ),
    widgetDragMonitor_( this )
{
    setWindowTitle( "Ssh Password" );
    setWindowFlags( Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint );

    widgetDragMonitor_.setEnabled( true );

    // opaque background
    setAttribute( Qt::WA_OpaquePaintEvent, true );
    setAttribute( Qt::WA_StyledBackground, false );
    setAttribute( Qt::WA_TranslucentBackground, true );

    // main layout
    QVBoxLayout* mainLayout = new QVBoxLayout();
    setLayout( mainLayout );
    layout()->setMargin(0);
    layout()->setSpacing(0);

    QHBoxLayout *hLayout( new QHBoxLayout() );
    hLayout->setMargin(0);
    layout()->addItem( hLayout );

    // pixmap
    QPixmap pixmap( ":/icons/32x32/dialog-password.png" );
    if( !pixmap.isNull() )
    {
        QLabel* label = new QLabel( this );
        label->setMargin(10);
        label->setPixmap( pixmap );
        hLayout->addWidget( label, 0, Qt::AlignHCenter );
    }

    hLayout->addWidget( editor_ = new AnimatedLineEditor( this ) );
    editor_->setEchoMode( QLineEdit::Password );
    editor_->setPlaceholderText( tr("Enter your SSH pass phrase here") );
    connect( editor_, SIGNAL(returnPressed()), SLOT(accept()) );

    // buttons
    hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    layout()->addItem( hLayout );

    // stretch
    hLayout->addStretch( 1 );
    QPushButton* button;

    // ok button
    hLayout->addWidget( button = new QPushButton( this ) );
    button->setIcon( QIcon( ":/icons/32x32/accept.png") );
    button->setText( tr( "OK" ) );
    button->setFlat( true );
    connect( button, SIGNAL(clicked()), SLOT(accept()) );

    hLayout->addWidget( button = new QPushButton( this ) );
    button->setIcon( QIcon( ":/icons/32x32/cancel.png") );
    button->setShortcut( Qt::Key_Escape );
    button->setText( tr( "Cancel" ) );
    button->setFlat( true );
    connect( button, SIGNAL(clicked()), SLOT(cancel()) );

    // initialize svg engine
    Svg::SvgEngine::get().reload();

    // margins
    Base::Margins margins( Svg::SvgEngine::get().margins() );
    setContentsMargins( margins.left(), margins.top(), margins.right(), margins.bottom() );
    _setMargins( margins );

    // outer padding
    _setOuterPadding( Svg::SvgEngine::get().outerPadding() );

    // resize
    QSize size( 400, minimumSizeHint().height() );
    resize( size );

    // move widget at desktop center
    QPoint position(
        ( ( QApplication::desktop()->width() - size.width() ) / 2 ),
        ( ( QApplication::desktop()->height() - size.height() ) / 2 ) );
    move( position );

}

//______________________________________________________
void AskPassQt::paintEvent( QPaintEvent* event )
{

    #if defined(Q_OS_WIN)

    QPixmap pixmap( size() );
    pixmap.fill( Qt::transparent );
    _paint( pixmap, event->rect() );
    WinUtil( this ).update( pixmap, 1 );

    #else

    _paint( *this, event->rect() );

    #endif

}

//______________________________________________________
void AskPassQt::_paint( QPaintDevice& device, QRect clip ) const
{

    // draw background
    QPainter painter( &device );

    #if !defined(Q_OS_WIN)
    painter.setClipRect( clip );
    #endif

    // draw background
    if( Transparency::CompositeEngine::get().isAvailable() )
    {

        painter.setRenderHints(QPainter::SmoothPixmapTransform);
        painter.setCompositionMode(QPainter::CompositionMode_Source );
        painter.fillRect( rect(), Qt::transparent );

    }

    // draw svg
    painter.drawPixmap( QPoint( 0, 0 ), Svg::SvgEngine::get().get( size() ) );
    painter.end();

    // loop over children and render them
    #if defined(Q_OS_WIN)
    foreach( QWidget* widget, findChildren<QWidget*>() )
    { widget->render( &device, widget->geometry().topLeft(), widget->rect().intersected( rect().translated( -widget->geometry().topLeft() ) ), 0 ); }
    #endif

}


//______________________________________________________
void AskPassQt::resizeEvent( QResizeEvent* event )
{
    setMask( RoundedRegion( rect(), RoundedRegion::All ) );

    // announce blur region
    Base::Margins outerPadding( Svg::SvgEngine::get().outerPadding() );
    if( outerPadding.isNull() )
    {
        outerPadding = Svg::SvgEngine::get().margins();
        outerPadding -= Base::Margins( 4 );
    }

    _updateBlurRegion( outerPadding.adjustedRect( rect() ) );

    return QWidget::resizeEvent( event );
}

//______________________________________________________
void AskPassQt::accept( void )
{
    puts( editor_->text().toUtf8() );
    close();
}

//______________________________________________________
void AskPassQt::cancel( void )
{ close(); }
